/*******************************************************************************
  MPLAB Harmony Application Source File
  
  Company:
    Microchip Technology Inc.
  
  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It 
    implements the logic of the application's state machine and it may call 
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include "app.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    
    Application strings and buffers are be defined outside this structure.
*/
/* Array in the RAM to store the data */
uint32_t scriptBuffer[APP_DEVICE_PAGE_SIZE_DIVIDED_BY_4] ALIGN(16);
uint32_t testBuffer[APP_DEVICE_PAGE_SIZE_DIVIDED_BY_4] ALIGN(16);

APP_DATA appData;

/* Driver objects.

  Summary:
    Contains driver objects.

  Description:
    This structure contains driver objects returned by the driver init routines
    to the application. These objects are passed to the driver tasks routines.
*/
APP_DRV_OBJECTS appDrvObject;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* TODO:  Add any necessary local functions.
*/

void PrintBuffer(bool ReadFlash)
{
    uint32_t x,y;
    uint8_t value1, value2, value3, value4;
    
    if (ReadFlash)
    {
        memcpy(testBuffer, (void *)APP_PROGRAM_FLASH_BASE_ADDRESS_VALUE, sizeof(scriptBuffer));
        SYS_PRINT("FLASH\r\n");
    }
    else
        SYS_PRINT("SCRIPT\r\n");                
        
    SYS_PRINT("         ---00--- ---01--- ---02--- ---03--- ---04--- ---05--- ---06--- ---07--- \r\n");
    for (x=0; x<1024; x++)
    {
        if (((x % 8) == 0) && (x>0))
        {
            SYS_PRINT("\r\n");
            SYS_PRINT("%8X ",x);
        }
        else
        {
            if (x == 0)
                SYS_PRINT("%8X ",x);
                
        }
        if (ReadFlash)
            y = testBuffer[x];
        else
            y = scriptBuffer[x];
        value1 = (uint8_t)((y & 0x000000FF));
        value2 = (uint8_t)((y & 0x0000FF00)>>8);
        value3 = (uint8_t)((y & 0x00FF0000)>>16);
        value4 = (uint8_t)((y & 0xFF000000)>>24);
        SYS_PRINT("%8X ",y);
    }
    SYS_PRINT("\r\n");
}

// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;
    appData.nvmState = APP_STATE_NVM_INIT;

    
    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}

/**********************************************************
 * FLASH NVM tasks routine. This function implements the
 * FLASH NVM state machine.
 ***********************************************************/
void APP_NVM_Tasks ( void )
{
   unsigned int x;
   uint32_t  bufOffset, addrOffset;
   
   x = APP_DATABUFF_SIZE;
   switch(appData.nvmState)
   {
      case APP_STATE_NVM_INIT:
        appData.nvmHandle = DRV_FLASH_Open(DRV_FLASH_INDEX_0, intent);
        appData.nvmState = APP_STATE_NVM_FILL_DATABUF_AND_ERASE_STATE;
        appData.nvmRowCount = APP_DEVICE_ROW_COUNT;
        PrintBuffer(true);
        break;

      case APP_STATE_NVM_FILL_DATABUF_AND_ERASE_STATE:
        for (x = 0; x < APP_DATABUFF_SIZE; x++)
        {
            scriptBuffer[x] = x;
        }
        LED_BLOff();
        LED_YLOff();

        PrintBuffer(false);
        /* Erase the page which consist of the row to be written */
        DRV_FLASH_ErasePage(appData.nvmHandle, APP_PROGRAM_FLASH_BASE_ADDRESS);
        appData.nvmCurrentRow = 0;              // set row value to zero
        appData.nvmState = APP_STATE_NVM_ERASE_COMPLETION_CHECK;
        break;

      case APP_STATE_NVM_ERASE_COMPLETION_CHECK:
        if(!DRV_FLASH_IsBusy(appData.nvmHandle))
        {
            PrintBuffer(true);
            appData.nvmState = APP_STATE_NVM_WRITE_START;
        }
        break;

      case APP_STATE_NVM_WRITE_START:
        /* Erase Success */
        /* Write a row of data to PROGRAM_FLASH_BASE_ADDRESS, using databuff array as the source */
        //PrintBuffer(true);
        bufOffset = appData.nvmCurrentRow * APP_DEVICE_ROW_SIZE_DIVIDED_BY_4;
        addrOffset = appData.nvmCurrentRow * APP_DEVICE_ROW_SIZE;
        DRV_FLASH_WriteRow(appData.nvmHandle, APP_PROGRAM_FLASH_BASE_ADDRESS + addrOffset, &scriptBuffer[bufOffset]);
        appData.nvmCurrentRow++;
        appData.nvmState = APP_STATE_NVM_WRITE_COMPLETION_CHECK_AND_VERIFY_CHECK;
        break;

      case APP_STATE_NVM_WRITE_COMPLETION_CHECK_AND_VERIFY_CHECK:
        if(!DRV_FLASH_IsBusy(appData.nvmHandle))
        {
            PrintBuffer(true);
            /* Verify that data written to flash memory is valid (databuff array read from kseg1) */
            if (!memcmp(&scriptBuffer[bufOffset], (void *)(APP_PROGRAM_FLASH_BASE_ADDRESS_VALUE + addrOffset), DRV_FLASH_ROW_SIZE))
            {
                appData.nvmState = APP_STATE_NVM_SUCCESS_STATE;
            }
            else
            {
                appData.nvmState = APP_STATE_NVM_ERROR_STATE;
            }
        }
        break;
         
      case APP_STATE_NVM_ERROR_STATE:
        /*stay here, nvm had a failure*/
        LED_BLOff();
        LED_YLOn();
        if (appData.nvmCurrentRow < APP_DEVICE_ROW_COUNT)
        {
            appData.nvmState = APP_STATE_NVM_WRITE_START;
        }
        else 
        {
            PrintBuffer(true);
            appData.nvmState = APP_STATE_NVM_IDLE_STATE;
        }
        break;

      case APP_STATE_NVM_SUCCESS_STATE:
        LED_BLOn();
        LED_YLOff();
        if (appData.nvmCurrentRow < APP_DEVICE_ROW_COUNT)
        {
            appData.nvmState = APP_STATE_NVM_WRITE_START;
        }
        else 
        {
            PrintBuffer(true);
            appData.nvmState = APP_STATE_NVM_IDLE_STATE;
        }
        break;

      case APP_STATE_NVM_IDLE_STATE:
        LED_BLToggle();
        LED_YLOff();
        break;
   }
} 


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        {
            bool appInitialized = true;
       
        
            if (appInitialized)
            {
            
                appData.state = APP_STATE_SERVICE_TASKS;
            }
            break;
        }

        case APP_STATE_SERVICE_TASKS:
        {
            APP_NVM_Tasks();
            break;
        }

        /* TODO: implement your application state machine.*/
        

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}

 

/*******************************************************************************
 End of File
 */
